﻿library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use IEEE.math_real.all;

entity example is
	Generic (
		RATIO		: Integer range   6 to  100 := 6
	);
	Port (
		ACLK	: in  std_logic;
		ARESETn	: in  std_logic;

		s_axi_AWVALID	: in	std_logic;
		s_axi_AWREADY	: out	std_logic;
		s_axi_AWADDR	: in	std_logic_vector(3 downto 0);
-- OMITTED	s_axi_AWPROT	: in	std_logic_vector(2 downto 0);
		s_axi_WVALID	: in	std_logic;
		s_axi_WREADY	: out	std_logic;
		s_axi_WDATA	: in	std_logic_vector(31 downto 0);
		s_axi_WSTRB	: in	std_logic_vector(3 downto 0);
		s_axi_BVALID	: out	std_logic;
		s_axi_BREADY	: in	std_logic;
-- OMITTED	s_axi_BRESP	: out	std_logic_vector(1 downto 0);
		s_axi_ARVALID	: in	std_logic;
		s_axi_ARREADY	: out	std_logic;
		s_axi_ARADDR	: in	std_logic_vector(3 downto 0);
-- OMITTED	s_axi_ARPROT	: in	std_logic_vector(2 downto 0);
		s_axi_RVALID	: out	std_logic;
		s_axi_RREADY	: in	std_logic;
		s_axi_RDATA	: out	std_logic_vector(31 downto 0);
-- OMITTED	s_axi_RRESP	: out	std_logic_vector(1 downto 0);

		s_axis_TVALID	: in	std_logic;
		s_axis_TREADY	: out	std_logic;
		s_axis_TDATA	: in	std_logic_vector(23 downto 0);
		s_axis_TSTRB	: in	std_logic_vector(2 downto 0);
-- OMITTED	s_axis_TKEEP	: in	std_logic_vector(2 downto 0);
		s_axis_TLAST	: in	std_logic;

		m_axis_TVALID	: out	std_logic;
		m_axis_TREADY	: in	std_logic;
		m_axis_TDATA	: out	std_logic_vector(15 downto 0);
		m_axis_TSTRB	: out	std_logic_vector(1 downto 0);
-- OMITTED	m_axis_TKEEP	: out	std_logic_vector(1 downto 0);
		m_axis_TLAST	: out	std_logic
	);
end example;

architecture Behavioral of example is
	constant DELTA		: Integer range  17 to 4095 := 17;
	signal set_s_ready	: std_logic := '0';
	signal is_dead_time	: std_logic := '0';
begin

	--! \brief	Stellt das gewünschte Verhältnis (`RATIO`) zwischen
	--!		FPGA- und Pixeltakt sicher.
	--!
	--! \details	Der Zähler beginnt zu zählen, sobald **ARESETn** `HIGH`
	--!		ist. Nach einer Anzahl von `RATIO` Takten werden die
	--!		Signale `set_s_ready` und `dead_time_clk` gesetzt.
	--!		Außerhalb der Totzeit wird ein neuer Zählzyklus erst
	--!		dann angestoßen, wenn ein Transfer erfolgt ist.
	--!
	--! \driven	- `set_s_ready`
	--!		- `dead_time_clk`
	ratio_counter : process (ARESETn, ACLK, s_data_transfer)
		variable counter : Integer range 0 to (RATIO + 1) := 1;
	begin
		if (ARESETn = '0') then
			counter := 1;
		elsif (rising_edge(ACLK)) then
			if (counter < RATIO) then
				counter := counter + 1;
			elsif (is_dead_time = '1') then
				counter := 1;
			elsif (s_data_transfer = '1') then
				counter := 1;
			end if;
			if (is_dead_time = '0') then
				if (counter = RATIO - 1) then
					set_s_ready <= '1';
				else
					set_s_ready <= '0';
				end if;
			end if;
			if (counter = RATIO) then
				dead_time_clk <= '1';
			else
				dead_time_clk <= '0';
			end if;
		end if;
	end process ratio_counter;

end Behavioral;
